@@ -128,6 +128,7 @@ public:
     PREPARE_RESULT_FAILURE,
     PREPARE_RESULT_OK,
     PREPARE_RESULT_DRMCHANGED,
+    PREPARE_RESULT_DRMUNCHANGED,
   };
 
   // Node definition
@@ -173,6 +174,7 @@ public:
     std::string codecs_;
     std::string codec_private_data_;
     std::string source_url_;
+    std::string base_url_;
     uint32_t bandwidth_;
     uint32_t samplingRate_;
     uint16_t width_, height_;
@@ -412,13 +414,13 @@ public:
   }*current_period_, *next_period_;
 
   std::vector<Period*> periods_;
-  std::string manifest_url_, base_url_, effective_url_, base_domain_, effective_domain_,
-      update_parameter_;
-  std::string::size_type update_parameter_pos_;
-  std::string etag_, last_modified_;
-  std::string media_renewal_url_;
-  uint32_t media_renewal_time_;
-  std::string manifest_parameter_;
+  std::string manifest_url_;
+  std::string base_url_;
+  std::string effective_url_;
+  std::string base_domain_;
+  std::string update_parameter_;
+  std::string etag_;
+  std::string last_modified_;
 
   /* XML Parsing*/
   XML_Parser parser_;
@@ -452,7 +454,8 @@ public:
   AdaptiveTree();
   virtual ~AdaptiveTree();
 
-  virtual bool open(const std::string &url, const std::string &manifestUpdateParam) = 0;
+  virtual bool open(const std::string& url, const std::string& manifestUpdateParam) = 0;
+  virtual bool open(const std::string& url, const std::string& manifestUpdateParam, std::map<std::string, std::string> additionalHeaders) = 0;
   virtual PREPARE_RESULT prepareRepresentation(Period* period,
                                                AdaptationSet* adp,
                                                Representation* rep,
@@ -483,19 +486,21 @@ public:
                : 0;
   };
 
-  void SetEffectiveURL(const std::string& url);
   std::string BuildDownloadUrl(const std::string& url) const;
 
   std::mutex &GetTreeMutex() { return treeMutex_; };
   bool HasUpdateThread() const { return updateThread_ != 0 && has_timeshift_buffer_ && updateInterval_ && !update_parameter_.empty(); };
   void RefreshUpdateThread();
   const std::chrono::time_point<std::chrono::system_clock> GetLastUpdated() const { return lastUpdated_; };
-  const std::chrono::time_point<std::chrono::system_clock> GetLastMediaRenewal() const { return lastMediaRenewal_; };
 
 protected:
-  virtual bool download(const char* url, const std::map<std::string, std::string> &manifestHeaders, void *opaque = nullptr, bool scanEffectiveURL = true);
+  virtual bool download(const char* url,
+                        const std::map<std::string, std::string>& manifestHeaders,
+                        void* opaque = nullptr,
+                        bool isManifest = true);
   virtual bool write_data(void *buffer, size_t buffer_size, void *opaque) = 0;
-  bool PreparePaths(const std::string &url, const std::string &manifestUpdateParam);
+  bool PreparePaths(const std::string &url);
+  void PrepareManifestUrl(const std::string &url, const std::string &manifestUpdateParam);
   void SortTree();
 
   // Live segment update section
@@ -507,7 +512,6 @@ protected:
   std::condition_variable updateVar_;
   std::thread *updateThread_;
   std::chrono::time_point<std::chrono::system_clock> lastUpdated_;
-  std::chrono::time_point<std::chrono::system_clock> lastMediaRenewal_;
 
 private:
   void SegmentUpdateWorker();
