@@ -90,9 +90,9 @@ static uint8_t GetChannels(const char** attr)
 static unsigned int ParseSegmentTemplate(const char** attr,
                                          std::string baseURL,
                                          std::string baseDomain,
-                                         DASHTree::SegmentTemplate& tpl)
+                                         DASHTree::SegmentTemplate& tpl,
+                                         unsigned int startNumber)
 {
-  unsigned int startNumber(1);
   for (; *attr;)
   {
     if (strcmp((const char*)*attr, "timescale") == 0)
@@ -482,9 +482,9 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
           {
             dash->current_representation_->segtpl_ = dash->current_adaptationset_->segtpl_;
 
-            dash->current_representation_->startNumber_ =
-                ParseSegmentTemplate(attr, dash->current_representation_->url_, dash->base_domain_,
-                                     dash->current_representation_->segtpl_);
+            dash->current_representation_->startNumber_ = ParseSegmentTemplate(
+                attr, dash->current_representation_->base_url_, dash->base_domain_,
+                dash->current_representation_->segtpl_, dash->current_adaptationset_->startNumber_);
             ReplacePlaceHolders(dash->current_representation_->segtpl_.media,
                                 dash->current_representation_->id,
                                 dash->current_representation_->bandwidth_);
@@ -604,9 +604,9 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
         }
         else if (strcmp(el, "SegmentTemplate") == 0)
         {
-          dash->current_adaptationset_->startNumber_ =
-              ParseSegmentTemplate(attr, dash->current_adaptationset_->base_url_,
-                                   dash->base_domain_, dash->current_adaptationset_->segtpl_);
+          dash->current_adaptationset_->startNumber_ = ParseSegmentTemplate(
+              attr, dash->current_adaptationset_->base_url_, dash->base_domain_,
+              dash->current_adaptationset_->segtpl_, dash->current_adaptationset_->startNumber_);
           dash->current_adaptationset_->timescale_ =
               dash->current_adaptationset_->segtpl_.timescale;
           dash->currentNode_ |= MPDNODE_SEGMENTTEMPLATE;
@@ -658,13 +658,14 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
           dash->current_representation_->timescale_ = dash->current_adaptationset_->timescale_;
           dash->current_representation_->duration_ = dash->current_adaptationset_->duration_;
           dash->current_representation_->startNumber_ = dash->current_adaptationset_->startNumber_;
-          dash->current_adaptationset_->representations_.push_back(dash->current_representation_);
           dash->current_representation_->width_ = dash->adpwidth_;
           dash->current_representation_->height_ = dash->adpheight_;
           dash->current_representation_->fpsRate_ = dash->adpfpsRate_;
           dash->current_representation_->fpsScale_ = dash->adpfpsScale_;
           dash->current_representation_->aspect_ = dash->adpaspect_;
           dash->current_representation_->containerType_ = dash->adpContainerType_;
+          dash->current_representation_->base_url_ = dash->current_adaptationset_->base_url_;
+          dash->current_adaptationset_->representations_.push_back(dash->current_representation_);
 
           dash->current_pssh_.clear();
           dash->current_hasRepURN_ = false;
@@ -957,9 +958,9 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
       }
       else if (strcmp(el, "SegmentTemplate") == 0)
       {
-        dash->current_period_->startNumber_ =
-            ParseSegmentTemplate(attr, dash->current_period_->base_url_, dash->base_domain_,
-                                 dash->current_period_->segtpl_);
+        dash->current_period_->startNumber_ = ParseSegmentTemplate(
+            attr, dash->current_period_->base_url_, dash->base_domain_,
+            dash->current_period_->segtpl_, dash->current_period_->startNumber_);
         dash->current_period_->timescale_ = dash->current_period_->segtpl_.timescale;
         dash->currentNode_ |= MPDNODE_SEGMENTTEMPLATE;
       }
@@ -1052,6 +1053,10 @@ static void XMLCALL start(void* data, const char* el, const char** attr)
       {
         uint64_t dur(0);
         AddDuration((const char*)*(attr + 1), dur, 1500);
+        // 0S minimumUpdatePeriod = refresh after every segment
+        // We already do that so lets set our minimum updateInterval to 30s
+        if (dur == 0)
+          dur = 30000;
         dash->SetUpdateInterval(static_cast<uint32_t>(dur));
       }
       attr += 2;
@@ -1113,6 +1118,8 @@ static void XMLCALL end(void* data, const char* el)
               else
                 url = dash->current_adaptationset_->base_url_ + dash->strXMLText_;
 
+              dash->current_representation_->base_url_ = url;
+
               if (dash->current_representation_->flags_ & AdaptiveTree::Representation::TEMPLATE)
               {
                 if (dash->current_representation_->flags_ &
@@ -1551,7 +1558,11 @@ static void XMLCALL end(void* data, const char* el)
 +---------------------------------------------------------------------*/
 bool DASHTree::open(const std::string& url, const std::string& manifestUpdateParam)
 {
-  PreparePaths(url, manifestUpdateParam);
+  return open(url, manifestUpdateParam, std::map<std::string, std::string>());
+}
+
+bool DASHTree::open(const std::string& url, const std::string& manifestUpdateParam, std::map<std::string, std::string> additionalHeaders)
+{
   parser_ = XML_ParserCreate(NULL);
   if (!parser_)
     return false;
@@ -1562,8 +1573,9 @@ bool DASHTree::open(const std::string& url, const std::string& manifestUpdatePar
   currentNode_ = 0;
   strXMLText_.clear();
 
-  std::string download_url = BuildDownloadUrl(manifest_url_);
-  bool ret = download(download_url.c_str(), manifest_headers_) && !periods_.empty();
+  PrepareManifestUrl(url, manifestUpdateParam);
+  additionalHeaders.insert(manifest_headers_.begin(), manifest_headers_.end());
+  bool ret = download(manifest_url_.c_str(), additionalHeaders) && !periods_.empty();
 
   XML_ParserFree(parser_);
   parser_ = 0;
@@ -1598,7 +1610,7 @@ void DASHTree::RefreshSegments(Period* period,
 {
   if ((type == VIDEO || type == AUDIO))
   {
-    lastUpdated_ = std::chrono::system_clock::now();
+    lastUpdated_ = GetTimePointNowTime();
     RefreshUpdateThread();
     RefreshLiveSegments();
   }
@@ -1612,8 +1624,9 @@ void DASHTree::RefreshLiveSegments()
     std::string replaced;
     uint32_t numReplace = ~0U;
     unsigned int nextStartNumber(~0);
+    std::string::size_type update_parameter_pos = update_parameter_.find("$START_NUMBER$");
 
-    if (~update_parameter_pos_)
+    if (~update_parameter_pos)
     {
       for (std::vector<Period*>::const_iterator bp(periods_.begin()), ep(periods_.end()); bp != ep;
            ++bp)
@@ -1634,10 +1647,14 @@ void DASHTree::RefreshLiveSegments()
           }
       Log(LOGLEVEL_DEBUG, "DASH Update: numReplace: %u, nextStartNumber: %u", numReplace,
           nextStartNumber);
+
+      if (update_parameter_[0] == '&' && manifest_url_.find("?") == std::string::npos)
+        update_parameter_[0] = '?';
+
       replaced = update_parameter_;
       char buf[32];
       sprintf(buf, "%u", nextStartNumber);
-      replaced.replace(update_parameter_pos_, 14, buf);
+      replaced.replace(update_parameter_pos, 14, buf);
     }
 
     DASHTree updateTree;
@@ -1646,10 +1663,8 @@ void DASHTree::RefreshLiveSegments()
     updateTree.supportedKeySystem_ = supportedKeySystem_;
     //Location element should be used on updates
     updateTree.location_ = location_;
-    updateTree.effective_url_ = effective_url_;
-    updateTree.effective_domain_ = effective_domain_;
 
-    if (!~update_parameter_pos_)
+    if (!~update_parameter_pos)
     {
       if (!etag_.empty())
         updateTree.manifest_headers_["If-None-Match"] = "\"" + etag_ + "\"";
@@ -1664,7 +1679,7 @@ void DASHTree::RefreshLiveSegments()
       location_ = updateTree.location_;
 
       //Youtube returns last smallest number in case the requested data is not available
-      if (~update_parameter_pos_ && updateTree.firstStartNumber_ < nextStartNumber)
+      if (~update_parameter_pos && updateTree.firstStartNumber_ < nextStartNumber)
         return;
 
       std::vector<Period*>::const_iterator bpd(periods_.begin()), epd(periods_.end());
@@ -1698,7 +1713,7 @@ void DASHTree::RefreshLiveSegments()
                 ;
               if (brd != erd && !(*br)->segments_.empty())
               {
-                if (~update_parameter_pos_) // partitial update
+                if (~update_parameter_pos) // partitial update
                 {
                   //Here we go -> Insert new segments
                   uint64_t ptsOffset = (*brd)->nextPts_ - (*br)->segments_[0]->startPTS_;
@@ -1742,11 +1757,23 @@ void DASHTree::RefreshLiveSegments()
                   if ((*br)->flags_ & DASHTree::Representation::TIMELINE)
                   {
                     uint64_t search_pts = (*br)->segments_[0]->range_begin_;
+                    uint64_t misaligned = 0;
                     for (const auto& s : (*brd)->segments_.data)
                     {
-                      if (s.range_begin_ >= search_pts)
+                      if (misaligned)
+                      {
+                        uint64_t ptsDiff = s.range_begin_ - (&s - 1)->range_begin_;
+                        // our misalignment is small ( < 2%), let's decrement the start number
+                        if (misaligned < (ptsDiff * 2 / 100))
+                          --(*brd)->startNumber_;
                         break;
-                      ++(*brd)->startNumber_;
+                      }
+                      if (s.range_begin_ == search_pts)
+                        break;
+                      else if (s.range_begin_ > search_pts)
+                        misaligned = search_pts - (&s - 1)->range_begin_;
+                      else 
+                        ++(*brd)->startNumber_;
                     }
                   }
                   else if ((*br)->segments_[0]->startPTS_ == (*brd)->segments_[0]->startPTS_)
