@@ -107,7 +107,7 @@ int HLSTree::processEncryption(std::string baseUrl, std::map<std::string, std::s
   if (map["METHOD"] == "AES-128" && !map["URI"].empty())
   {
     current_pssh_ = map["URI"];
-    if (current_pssh_[0] != '/' && current_pssh_.find("://", 0) == std::string::npos)
+    if (current_pssh_[0] != '/' && current_pssh_.find("://") == std::string::npos)
       current_pssh_ = baseUrl + current_pssh_;
 
     current_iv_ = m_decrypter->convertIV(map["IV"]);
@@ -158,13 +158,19 @@ int HLSTree::processEncryption(std::string baseUrl, std::map<std::string, std::s
 
 bool HLSTree::open(const std::string& url, const std::string& manifestUpdateParam)
 {
-  PreparePaths(url, manifestUpdateParam);
-  if (download(manifest_url_.c_str(), manifest_headers_, &manifest_stream))
-    return processManifest(manifest_stream, url);
+  return open(url, manifestUpdateParam, std::map<std::string, std::string>());
+}
+
+bool HLSTree::open(const std::string& url, const std::string& manifestUpdateParam, std::map<std::string, std::string> additionalHeaders)
+{
+  PrepareManifestUrl(url, manifestUpdateParam);
+  additionalHeaders.insert(manifest_headers_.begin(), manifest_headers_.end());
+  if (download(manifest_url_.c_str(), additionalHeaders, &manifest_stream))
+    return processManifest(manifest_stream);
   return false;
 }
 
-bool HLSTree::processManifest(std::stringstream& stream, const std::string& url)
+bool HLSTree::processManifest(std::stringstream& stream)
 {
 #if FILEDEBUG
   FILE* f = fopen("inputstream_adaptive_master.m3u8", "w");
@@ -232,10 +238,7 @@ bool HLSTree::processManifest(std::stringstream& stream, const std::string& url)
       std::map<std::string, std::string>::iterator res;
       if ((res = map.find("URI")) != map.end())
       {
-        if (res->second[0] != '/' && res->second.find("://", 0) == std::string::npos)
-          rep->source_url_ = base_url_ + res->second;
-        else
-          rep->source_url_ = res->second;
+        rep->source_url_ = BuildDownloadUrl(res->second);
 
         // default to WebVTT
         if (type == SUBTITLE)
@@ -306,7 +309,7 @@ bool HLSTree::processManifest(std::stringstream& stream, const std::string& url)
       current_representation_->bandwidth_ = 0;
       current_representation_->codecs_ = getVideoCodec("");
       current_representation_->containerType_ = CONTAINERTYPE_NOTYPE;
-      current_representation_->source_url_ = url;
+      current_representation_->source_url_ = manifest_url_;
       current_adaptationset_->representations_.push_back(current_representation_);
 
       // We assume audio is included
@@ -316,10 +319,7 @@ bool HLSTree::processManifest(std::stringstream& stream, const std::string& url)
     }
     else if (!line.empty() && line.compare(0, 1, "#") != 0 && current_representation_)
     {
-      if (line[0] != '/' && line.find("://", 0) == std::string::npos)
-        current_representation_->source_url_ = base_url_ + line;
-      else
-        current_representation_->source_url_ = line;
+      current_representation_->source_url_ = BuildDownloadUrl(line);
 
       //Ignore duplicate reps
       for (auto const* rep : current_adaptationset_->representations_)
@@ -394,7 +394,6 @@ HLSTree::PREPARE_RESULT HLSTree::prepareRepresentation(Period* period,
     Segment newInitialization;
     uint32_t segmentId(rep->getCurrentSegmentNumber());
     std::stringstream stream;
-    std::string download_url = BuildDownloadUrl(rep->source_url_);
     uint32_t adp_pos =
         std::find(period->adaptationSets_.begin(), period->adaptationSets_.end(), adp) -
         period->adaptationSets_.begin();
@@ -407,7 +406,7 @@ HLSTree::PREPARE_RESULT HLSTree::prepareRepresentation(Period* period,
 
     if (rep->flags_ & Representation::DOWNLOADED)
       ;
-    else if (download(download_url.c_str(), manifest_headers_, &stream, false))
+    else if (download(rep->source_url_.c_str(), manifest_headers_, &stream, false))
     {
 #if FILEDEBUG
       FILE* f = fopen("inputstream_adaptive_sub.m3u8", "w");
@@ -434,9 +433,9 @@ HLSTree::PREPARE_RESULT HLSTree::prepareRepresentation(Period* period,
       segment.startPTS_ = ~0ULL;
       segment.pssh_set_ = 0;
 
-      std::string::size_type paramPos = rep->source_url_.find('?');
+      std::string::size_type paramPos = effective_url_.find('?');
       base_url =
-          (paramPos == std::string::npos) ? rep->source_url_ : rep->source_url_.substr(0, paramPos);
+          (paramPos == std::string::npos) ? effective_url_ : effective_url_.substr(0, paramPos);
 
       paramPos = base_url.rfind('/');
       if (paramPos != std::string::npos)
@@ -485,7 +484,8 @@ HLSTree::PREPARE_RESULT HLSTree::prepareRepresentation(Period* period,
                 rep->containerType_ = CONTAINERTYPE_ADTS;
               else if (strncmp(line.c_str() + ext, ".mp4", 4) == 0)
                 rep->containerType_ = CONTAINERTYPE_MP4;
-              else if (strncmp(line.c_str() + ext, ".vtt", 4) == 0)
+              else if (strncmp(line.c_str() + ext, ".vtt", 4) == 0 ||
+                       strncmp(line.c_str() + ext, ".webvtt", 7) == 0)
                 rep->containerType_ = CONTAINERTYPE_TEXT;
               else
               {
@@ -503,7 +503,7 @@ HLSTree::PREPARE_RESULT HLSTree::prepareRepresentation(Period* period,
           if (!byteRange || rep->url_.empty())
           {
             std::string url;
-            if (line[0] != '/' && line.find("://", 0) == std::string::npos)
+            if (line[0] != '/' && line.find("://") == std::string::npos)
               url = base_url + line;
             else
               url = line;
@@ -618,11 +618,10 @@ HLSTree::PREPARE_RESULT HLSTree::prepareRepresentation(Period* period,
           segment.range_end_ = 0;
           segment.startPTS_ = ~0ULL;
           segment.pssh_set_ = 0;
-          pts = 0;
 
           if (currentEncryptionType == ENCRYPTIONTYPE_WIDEVINE)
           {
-            rep->pssh_set_ = insert_psshset(NOTYPE, period, adp);
+            rep->pssh_set_ = insert_psshset(adp->type_, period, adp);
             period->encryptionState_ |= ENCRYTIONSTATE_SUPPORTED;
           }
 
@@ -646,9 +645,11 @@ HLSTree::PREPARE_RESULT HLSTree::prepareRepresentation(Period* period,
             case ENCRYPTIONTYPE_WIDEVINE:
               currentEncryptionType = ENCRYPTIONTYPE_WIDEVINE;
               period->encryptionState_ |= ENCRYTIONSTATE_SUPPORTED;
-              rep->pssh_set_ = insert_psshset(NOTYPE, period, adp);
-              if (period->psshSets_[rep->pssh_set_].use_count_ == 1)
-                retVal = PREPARE_RESULT_DRMCHANGED;
+              rep->pssh_set_ = insert_psshset(adp->type_, period, adp);
+              retVal = period->psshSets_[rep->pssh_set_].use_count_ == 1 ||
+                               retVal == PREPARE_RESULT_DRMCHANGED
+                           ? PREPARE_RESULT_DRMCHANGED
+                           : PREPARE_RESULT_DRMUNCHANGED;
               break;
             default:
               break;
@@ -671,7 +672,7 @@ HLSTree::PREPARE_RESULT HLSTree::prepareRepresentation(Period* period,
               delete[] newInitialization.url;
             segmentInitialization = true;
             std::string uri = map["URI"];
-            if (uri[0] != '/' && uri.find("://", 0) == std::string::npos)
+            if (uri[0] != '/' && uri.find("://") == std::string::npos)
               map_url = base_url + uri;
             else
               map_url = uri;
